<?php
/**
 * Widget and Display classes for the Motorsports Calendar Widget plugin
 *
 * @package Motorsports_Calendar_Widget
 */

// If this file is called directly, abort.
if (!defined('ABSPATH')) {
    die;
}

/**
 * Class to handle shared functionality between widget and shortcode
 */
class Motocalwid_Display {
    /**
     * Generate the calendar HTML
     *
     * @param array $args Display arguments
     * @return string Generated HTML
     */
    public static function generate_calendar_html($args) {
        $defaults = array(
            'series' => 'all',
            'theme' => 'light',
            'display' => 'compact',
            'color' => '#1a1a1a',
            'weather_display' => 'no',
            'weather_bg_color' => 'rgba(0, 0, 0, 0.05)',
            'weather_text_color' => 'inherit',
            'time_size' => '1.5em',
            'label_size' => '0.8em'
        );

        $args = wp_parse_args($args, $defaults);
        
        // Adjust weather background color based on theme if not explicitly set
        if ($args['weather_bg_color'] === 'rgba(0, 0, 0, 0.05)' && $args['theme'] === 'dark') {
            $args['weather_bg_color'] = 'rgba(255, 255, 255, 0.05)';
        }
        
        $html = '<div id="motorsports-calendar-countdown-' . esc_attr(uniqid()) . '" ';
        $html .= 'data-series="' . esc_attr($args['series']) . '" ';
        $html .= 'data-theme="' . esc_attr($args['theme']) . '" ';
        $html .= 'data-display="' . esc_attr($args['display']) . '" ';
        $html .= 'data-color="' . esc_attr($args['color']) . '" ';
        $html .= 'data-weather-display="' . esc_attr($args['weather_display']) . '" ';
        $html .= 'data-weather-bg-color="' . esc_attr($args['weather_bg_color']) . '" ';
        $html .= 'data-weather-text-color="' . esc_attr($args['weather_text_color']) . '" ';
        $html .= 'data-time-size="' . esc_attr($args['time_size']) . '" ';
        $html .= 'data-label-size="' . esc_attr($args['label_size']) . '">';
        $html .= '</div>';
        
        return $html;
    }

    /**
     * Ensure required script is enqueued
     */
    public static function enqueue_required_script() {
        wp_enqueue_script(
            'motocalwid-calendar', 
            'https://motorsportscalendar.com/widgets/countdown.js', 
            array(), 
            MOTOCALWID_VERSION,
            true
        );
    }
}

/**
 * Widget Class
 */
class Motocalwid_Widget extends WP_Widget {
    /**
     * Setup the widget name, description, etc.
     */
    public function __construct() {
        $widget_ops = array(
            'classname' => 'motocalwid_widget',
            'description' => esc_html__('Display countdown timers for motorsport events', 'motorsports-calendar-widget')
        );
        parent::__construct(
            'motocalwid_widget', 
            esc_html__('Motorsports Calendar', 'motorsports-calendar-widget'), 
            $widget_ops
        );
    }

    /**
     * Frontend display of widget
     *
     * @param array $args     Widget arguments
     * @param array $instance Saved values from database
     */
    public function widget($args, $instance) {
        echo wp_kses_post($args['before_widget']);
        
        if (!empty($instance['title'])) {
            echo wp_kses_post($args['before_title']) . 
                 wp_kses_post(apply_filters('widget_title', $instance['title'])) . 
                 wp_kses_post($args['after_title']);
        }

        echo wp_kses_post(Motocalwid_Display::generate_calendar_html($instance));
        Motocalwid_Display::enqueue_required_script();

        echo wp_kses_post($args['after_widget']);
    }

    /**
     * Backend widget form
     *
     * @param array $instance Previously saved values from database
     */
    public function form($instance) {
        $title = !empty($instance['title']) ? $instance['title'] : '';
        $series = !empty($instance['series']) ? $instance['series'] : 'all';
        $theme = !empty($instance['theme']) ? $instance['theme'] : 'light';
        $display = !empty($instance['display']) ? $instance['display'] : 'compact';
        $color = !empty($instance['color']) ? $instance['color'] : '#1a1a1a';
        $weather = !empty($instance['weather_display']) ? $instance['weather_display'] : 'no';
        $weather_bg = !empty($instance['weather_bg_color']) ? $instance['weather_bg_color'] : 'rgba(0, 0, 0, 0.05)';
        $weather_text = !empty($instance['weather_text_color']) ? $instance['weather_text_color'] : 'inherit';
        $time_size = !empty($instance['time_size']) ? str_replace('em', '', $instance['time_size']) : '1.5';
        $label_size = !empty($instance['label_size']) ? str_replace('em', '', $instance['label_size']) : '0.8';

        // Enqueue WordPress color picker
        wp_enqueue_style('wp-color-picker');
        wp_enqueue_script('wp-color-picker');
        
        // Add small inline script to initialize color pickers
        ?>
        <script>
            jQuery(document).ready(function($) {
                $('.color-picker').wpColorPicker();
            });
        </script>
        <?php

        // Series options - organized by category
        $series_options = array(
            'all' => esc_html__('All Series', 'motorsports-calendar-widget'),
            // Formula Racing
            'f1' => esc_html__('Formula 1', 'motorsports-calendar-widget'),
            'f2' => esc_html__('Formula 2', 'motorsports-calendar-widget'),
            'f3' => esc_html__('Formula 3', 'motorsports-calendar-widget'),
            'f1-academy' => esc_html__('F1 Academy', 'motorsports-calendar-widget'),
            'formula-e' => esc_html__('Formula E', 'motorsports-calendar-widget'),
            'super-formula' => esc_html__('Super Formula', 'motorsports-calendar-widget'),
            // IndyCar
            'indycar' => esc_html__('IndyCar Series', 'motorsports-calendar-widget'),
            'indylights' => esc_html__('Indy NXT', 'motorsports-calendar-widget'),
            // Endurance & GT Racing
            'wec' => esc_html__('World Endurance Championship', 'motorsports-calendar-widget'),
            'lemans' => esc_html__('24 Hours of Le Mans', 'motorsports-calendar-widget'),
            'imsa' => esc_html__('IMSA SportsCar Championship', 'motorsports-calendar-widget'),
            'intercontinental-gt-challenge' => esc_html__('Intercontinental GT Challenge', 'motorsports-calendar-widget'),
            'aslms' => esc_html__('Asian Le Mans Series', 'motorsports-calendar-widget'),
            // NASCAR
            'nascar-cup' => esc_html__('NASCAR Cup Series', 'motorsports-calendar-widget'),
            'nascar-xs' => esc_html__('NASCAR Xfinity Series', 'motorsports-calendar-widget'),
            'nascar-truck' => esc_html__('NASCAR Truck Series', 'motorsports-calendar-widget'),
            'nascar-euro' => esc_html__('NASCAR Euro Series', 'motorsports-calendar-widget'),
            // Touring Cars
            'dtm' => esc_html__('DTM', 'motorsports-calendar-widget'),
            'v8supercars' => esc_html__('Supercars Championship', 'motorsports-calendar-widget'),
            'supergt' => esc_html__('Super GT', 'motorsports-calendar-widget'),
            // Motorcycles
            'motogp' => esc_html__('MotoGP', 'motorsports-calendar-widget'),
            'moto2' => esc_html__('Moto2', 'motorsports-calendar-widget'),
            'moto3' => esc_html__('Moto3', 'motorsports-calendar-widget'),
            'wsbk' => esc_html__('World Superbike', 'motorsports-calendar-widget'),
            // Rally & Off-Road
            'wrc' => esc_html__('World Rally Championship', 'motorsports-calendar-widget'),
            'dakar' => esc_html__('Dakar Rally', 'motorsports-calendar-widget'),
        );

        ?>
        <p>
            <label for="<?php echo esc_attr($this->get_field_id('title')); ?>">
                <?php esc_html_e('Title:', 'motorsports-calendar-widget'); ?>
            </label>
            <input class="widefat" 
                   id="<?php echo esc_attr($this->get_field_id('title')); ?>"
                   name="<?php echo esc_attr($this->get_field_name('title')); ?>"
                   type="text"
                   value="<?php echo esc_attr($title); ?>">
        </p>

        <p>
            <label for="<?php echo esc_attr($this->get_field_id('series')); ?>">
                <?php esc_html_e('Series:', 'motorsports-calendar-widget'); ?>
            </label>
            <select class="widefat"
                    id="<?php echo esc_attr($this->get_field_id('series')); ?>"
                    name="<?php echo esc_attr($this->get_field_name('series')); ?>">
                <?php foreach ($series_options as $value => $label) : ?>
                    <option value="<?php echo esc_attr($value); ?>"
                            <?php selected($series, $value); ?>>
                        <?php echo esc_html($label); ?>
                    </option>
                <?php endforeach; ?>
            </select>
        </p>

        <p>
            <label for="<?php echo esc_attr($this->get_field_id('theme')); ?>">
                <?php esc_html_e('Theme:', 'motorsports-calendar-widget'); ?>
            </label>
            <select class="widefat"
                    id="<?php echo esc_attr($this->get_field_id('theme')); ?>"
                    name="<?php echo esc_attr($this->get_field_name('theme')); ?>">
                <option value="light" <?php selected($theme, 'light'); ?>>
                    <?php esc_html_e('Light', 'motorsports-calendar-widget'); ?>
                </option>
                <option value="dark" <?php selected($theme, 'dark'); ?>>
                    <?php esc_html_e('Dark', 'motorsports-calendar-widget'); ?>
                </option>
            </select>
        </p>

        <p>
            <label for="<?php echo esc_attr($this->get_field_id('display')); ?>">
                <?php esc_html_e('Display Style:', 'motorsports-calendar-widget'); ?>
            </label>
            <select class="widefat"
                    id="<?php echo esc_attr($this->get_field_id('display')); ?>"
                    name="<?php echo esc_attr($this->get_field_name('display')); ?>">
                <option value="compact" <?php selected($display, 'compact'); ?>>
                    <?php esc_html_e('Compact', 'motorsports-calendar-widget'); ?>
                </option>
                <option value="full" <?php selected($display, 'full'); ?>>
                    <?php esc_html_e('Full', 'motorsports-calendar-widget'); ?>
                </option>
            </select>
        </p>

        <p>
			<label for="<?php echo esc_attr($this->get_field_id('color')); ?>">
				<?php esc_html_e('Accent Color:', 'motorsports-calendar-widget'); ?>
			</label>
			<input class="widefat" 
				   id="<?php echo esc_attr($this->get_field_id('color')); ?>"
				   name="<?php echo esc_attr($this->get_field_name('color')); ?>"
				   type="color"
				   value="<?php echo esc_attr($color); ?>">
		</p>

        <p>
            <label for="<?php echo esc_attr($this->get_field_id('weather_display')); ?>">
                <?php esc_html_e('Weather Display:', 'motorsports-calendar-widget'); ?>
            </label>
            <select class="widefat"
                    id="<?php echo esc_attr($this->get_field_id('weather_display')); ?>"
                    name="<?php echo esc_attr($this->get_field_name('weather_display')); ?>">
                <option value="no" <?php selected($weather, 'no'); ?>>
                    <?php esc_html_e('No Weather', 'motorsports-calendar-widget'); ?>
                </option>
                <option value="minimal" <?php selected($weather, 'minimal'); ?>>
                    <?php esc_html_e('Minimal', 'motorsports-calendar-widget'); ?>
                </option>
                <option value="full" <?php selected($weather, 'full'); ?>>
                    <?php esc_html_e('Full', 'motorsports-calendar-widget'); ?>
                </option>
            </select>
        </p>

        <p>
            <label for="<?php echo esc_attr($this->get_field_id('weather_bg_color')); ?>">
                <?php esc_html_e('Weather Background Color:', 'motorsports-calendar-widget'); ?>
            </label>
            <input class="widefat"
                   id="<?php echo esc_attr($this->get_field_id('weather_bg_color')); ?>"
                   name="<?php echo esc_attr($this->get_field_name('weather_bg_color')); ?>"
                   type="color"
                   value="<?php echo esc_attr($weather_bg); ?>">
        </p>

        <p>
            <label for="<?php echo esc_attr($this->get_field_id('weather_text_color')); ?>">
                <?php esc_html_e('Weather Text Color:', 'motorsports-calendar-widget'); ?>
            </label>
            <input class="widefat"
                   id="<?php echo esc_attr($this->get_field_id('weather_text_color')); ?>"
                   name="<?php echo esc_attr($this->get_field_name('weather_text_color')); ?>"
                   type="color"
                   value="<?php echo esc_attr($weather_text); ?>">
        </p>

        <p>
            <label for="<?php echo esc_attr($this->get_field_id('time_size')); ?>">
                <?php esc_html_e('Time Font Size (em):', 'motorsports-calendar-widget'); ?>
            </label>
            <input class="tiny-text"
                   id="<?php echo esc_attr($this->get_field_id('time_size')); ?>"
                   name="<?php echo esc_attr($this->get_field_name('time_size')); ?>"
                   type="number"
                   step="0.1"
                   min="0.5"
                   max="3"
                   value="<?php echo esc_attr($time_size); ?>">
        </p>

        <p>
            <label for="<?php echo esc_attr($this->get_field_id('label_size')); ?>">
                <?php esc_html_e('Label Font Size (em):', 'motorsports-calendar-widget'); ?>
            </label>
            <input class="tiny-text"
                   id="<?php echo esc_attr($this->get_field_id('label_size')); ?>"
                   name="<?php echo esc_attr($this->get_field_name('label_size')); ?>"
                   type="number"
                   step="0.1"
                   min="0.5"
                   max="2"
                   value="<?php echo esc_attr($label_size); ?>">
        </p>
        <?php
    }

    /**
     * Sanitize widget form values as they are saved
     *
     * @param array $new_instance Values just sent to be saved
     * @param array $old_instance Previously saved values from database
     * @return array Updated safe values to be saved
     */
    public function update($new_instance, $old_instance) {
        $instance = array();
        $instance['title'] = (!empty($new_instance['title'])) ? 
            sanitize_text_field($new_instance['title']) : '';
        $instance['series'] = (!empty($new_instance['series'])) ? 
            sanitize_text_field($new_instance['series']) : 'all';
        $instance['theme'] = (!empty($new_instance['theme'])) ? 
            sanitize_text_field($new_instance['theme']) : 'light';
        $instance['display'] = (!empty($new_instance['display'])) ? 
            sanitize_text_field($new_instance['display']) : 'compact';
        $instance['color'] = (!empty($new_instance['color'])) ? 
            sanitize_hex_color($new_instance['color']) : '#1a1a1a';
        $instance['weather_display'] = (!empty($new_instance['weather_display'])) ? 
            sanitize_text_field($new_instance['weather_display']) : 'no';
        $instance['weather_bg_color'] = (!empty($new_instance['weather_bg_color'])) ? 
            sanitize_text_field($new_instance['weather_bg_color']) : 'rgba(0, 0, 0, 0.05)';
        $instance['weather_text_color'] = (!empty($new_instance['weather_text_color'])) ? 
            sanitize_text_field($new_instance['weather_text_color']) : 'inherit';
        
        // Ensure font sizes are valid numbers and append 'em'
        $time_size = (!empty($new_instance['time_size'])) ? 
            floatval($new_instance['time_size']) : 1.5;
        $time_size = max(0.5, min(3, $time_size)); // Clamp between 0.5 and 3
        $instance['time_size'] = $time_size . 'em';

        $label_size = (!empty($new_instance['label_size'])) ? 
            floatval($new_instance['label_size']) : 0.8;
        $label_size = max(0.5, min(2, $label_size)); // Clamp between 0.5 and 2
        $instance['label_size'] = $label_size . 'em';
        
        return $instance;
    }
}

/**
 * Shortcode handler for the motorsports calendar
 *
 * @param array $atts Shortcode attributes
 * @return string Generated HTML
 */
function motocalwid_shortcode($atts) {
    // Parse shortcode attributes
    $atts = shortcode_atts(
        array(
            'series' => 'all',
            'theme' => 'light',
            'display' => 'compact',
            'color' => '#1a1a1a',
            'weather_display' => 'no',
            'weather_bg_color' => 'rgba(0, 0, 0, 0.05)',
            'weather_text_color' => 'inherit',
            'time_size' => '1.5em',
            'label_size' => '0.8em'
        ),
        $atts,
        'motorsports_calendar'
    );

    // Use shared functionality
    Motocalwid_Display::enqueue_required_script();
    return Motocalwid_Display::generate_calendar_html($atts);
}

// Register shortcode
add_shortcode('motorsports_calendar', 'motocalwid_shortcode');